/*******************************************************************************

libunits.h

1997 R3 Software Ltd. / Timothy Wayper

Library to facilitate translation between different unit systems.
WARNING: this is the first, VERY sketchy, draft API. Don't rely on it.

$Id: libunits.h,v 1.2 1997/09/21 05:33:56 timmy Exp $

*******************************************************************************/


#ifndef _LIBUNITS_H
#define _LIBUNITS_H


/* BeOS Includes */

#include <SupportDefs.h>

#pragma export on


/* Constants */

#define UNIT_TYPE_NAME_LENGTH		32
#define UNIT_TYPE_DEF_LENGTH		32

#define UNIT_FAMILY_NAME_LENGTH	32

#define UNIT_INFO_NAME_LENGTH		32


/* Data structure definitions */

/*
Notes:
For definition strings, the primitive types are currently:

	Primitive	Symbol	Units
	distance	d		metres
	time		t		standard earth seconds
	mass		M		kilograms

(more primitive types (eg energy) will come later)
Valid operations (higher precedance first) are: ( ) ^ * /

Eg,
type "distance" is defined as "d"
type "area" is defined as "d^2"
type "acceleration" is defined as "d/t^2"
*/
struct UNITTypeInfo {
	char name[UNIT_TYPE_NAME_LENGTH];
	char def[UNIT_TYPE_DEF_LENGTH];
};

/* note: there will normally only be one default family */
struct UNITFamilyInfo {
	char name[UNIT_FAMILY_NAME_LENGTH];
	bool defaultFlag; /* true if this is a default family */
};

/* maybe this should support more than just linear units? */
struct UNITInfo {
	char name[UNIT_INFO_NAME_LENGTH];		/* name, eg, "foot" */
	char plural[UNIT_INFO_NAME_LENGTH];	/* plural form eg, "feet" */
	char abbrev[UNIT_INFO_NAME_LENGTH];	/* abbreviated form, eg, "ft" */
	char family[UNIT_FAMILY_NAME_LENGTH];	/* family unit belongs to, eg, "imperial" */
	char type[UNIT_TYPE_NAME_LENGTH];		/* type of unit, eg, "distance" */
	double factor;	/* multiplicative relative to base units */
	double offset;	/* additive relative to base units */
	bool defaultFlag;	/* true is this is the default unit for this _type_ */
};

/* Version Constants */

#define UNIT_CURRENT_VERSION	2	/* this version */
#define UNIT_MIN_VERSION		1	/* last version with equivalent API */


#ifdef __cplusplus
extern "C" {
#endif


/* API */

	/* returns version string
	Test the dynamically linked address of this function to see whether
	libunits is loaded. */
	extern const char *UNITVersion(
		int32 & outCurVersion,			/* current version spoken */
		int32 & outMinVersion);			/* minimum version understood */
	
	/* returns current user's default unit family (eg "metric") */
	extern status_t UNITGetDefaultFamily(
		UNITFamilyInfo &	outInfo		/* UNITFamilyInfo to write into */
		);
		
	/* returns list of all unit families ("SI", "metric", "imperial" etc.)
	call delete [] on outInfo after you're done */
	extern status_t UNITGetAllFamilies(
		UNITFamilyInfo * &	outInfo,	/* array of UNITFamilyInfo */
		uint32 &			outCount	/* item count of array */
		);
	
	/* returns list of all unit types ("distance", "area", "time" etc.)
	call delete [] on outInfo after you're done */
	extern status_t UNITGetAllTypes(
		UNITTypeInfo * &	outInfo,	/* array of UNITTypeInfo */
		uint32 &			outCount	/* item count of array */
		);

	/* returns list of all units ("metres", "hours", "hectares" etc.)
	I doubt you would want to use this very often
	call delete [] on outInfo after you're done */
	extern status_t UNITGetAllUnits(
		UNITInfo * &		outInfo,	/* array of UNITInfo */
		uint32 &			outCount	/* item count of array */
		);

	/* returns list of all units of a particular *type*
	call delete [] on outInfo after you're done */
	extern status_t UNITGetUnitsByType(
		UNITInfo * &		outInfo,	/* array of UNITInfo */
		uint32 &			outCount,	/* item count of array */
		const char *		type		/* type of unit to get */
		);

	/* returns list of all units in a particular *family*
	call delete [] on outInfo after you're done */
	extern status_t UNITGetUnitsByFamily(
		UNITInfo * &		outInfo,	/* array of UNITInfo */
		uint32 &			outCount,	/* item count of array */
		const char *		family		/* family of unit to get */
		);

	/* converts between two units of the *same* type */
	extern status_t UNITConvert(
		const double		from,		/* value to convert */
		double &			outTo,		/* out: converted value */
		const UNITInfo &	fromType,	/* unit of input from */
		const UNITInfo &	toType		/* unit of output outTo */
		);

	/* debug: dumps UNITFamilyInfo to stdout. */
	extern void UNITDumpFamilyInfo(
		const UNITFamilyInfo &	info	/* family info to dump */
		);
	
	/* debug: dumps UNITTypeInfo to stdout. */
	extern void UNITDumpTypeInfo(
		const UNITTypeInfo & info		/* type info to dump */
		);
	
	/* debug: dumps UNITInfo to stdout. */
	extern void UNITDumpUnitInfo(
		const UNITInfo &	info		/* unit info to dump */
		);

#ifdef __cplusplus
}
#endif

#pragma export reset


/******************************************************************************/

#endif /* _LIBUNITS_H */

/* end libunits.h */